import docopt


class ValidationException(Exception):
    pass


def transform(args, transforms):
    for key, function in transforms.items():
        try:
            arg = args[key]
        except KeyError:
            raise ValidationException(
                '%s is not a valid argument for transformation.' % key
            )

        try:
            arg = function(arg)
        except Exception:
            raise ValidationException(
                'Could not transform parameter %s.' % key
            )

        args[key] = arg

def validate(args, validations):
    for key, function in validations.items():
        try:
            arg = args[key]
        except KeyError:
            raise ValidationException(
                '%s is not a valid argument for validation.' % key
            )

        try:
            success = function(arg)
            if not success:
                raise Exception
        except Exception:
            raise ValidationException(
                'Could not validate parameter %s.' % key
            )

def docopt_wrapper(
        doc,
        argv=None,
        help_=True,
        version=None,
        options_first=False,
        transforms=None,
        validations=None
    ):
    args = docopt.docopt(doc, argv, help_, version, options_first)


    if validations:
        validate(args, validations)

    if transforms:
        transform(args, transforms)


    return args
